@@ -2,6 +2,19 @@ from __future__ import unicode_literals
 
 import sys
 import threading
+import time
+import pytest
+
+from prometheus_client import (
+    CollectorRegistry, CONTENT_TYPE_LATEST, Counter, delete_from_gateway, Enum,
+    Gauge, generate_latest, Histogram, Info, instance_ip_grouping_key, Metric,
+    push_to_gateway, pushadd_to_gateway, Summary,
+)
+from prometheus_client.core import GaugeHistogramMetricFamily, Timestamp
+from prometheus_client.exposition import (
+    basic_auth_handler, default_handler, MetricsHandler, generate_latest,
+)
+from prometheus_client import core
 
 if sys.version_info < (2, 7):
     # We need the skip decorators from unittest2 on Python 2.6.
@@ -9,11 +22,6 @@ if sys.version_info < (2, 7):
 else:
     import unittest
 
-from prometheus_client import Gauge, Counter, Summary, Histogram, Metric
-from prometheus_client import CollectorRegistry, generate_latest
-from prometheus_client import push_to_gateway, pushadd_to_gateway, delete_from_gateway
-from prometheus_client import CONTENT_TYPE_LATEST, instance_ip_grouping_key
-from prometheus_client.exposition import default_handler, basic_auth_handler, MetricsHandler
 
 try:
     from BaseHTTPServer import BaseHTTPRequestHandler
@@ -28,10 +36,38 @@ class TestGenerateText(unittest.TestCase):
     def setUp(self):
         self.registry = CollectorRegistry()
 
+        # Mock time so _created values are fixed.
+        self.old_time = time.time
+        time.time = lambda: 123.456
+
+    def tearDown(self):
+        time.time = self.old_time
+
+    def custom_collector(self, metric_family):
+        class CustomCollector(object):
+            def collect(self):
+                return [metric_family]
+        self.registry.register(CustomCollector())
+
     def test_counter(self):
         c = Counter('cc', 'A counter', registry=self.registry)
         c.inc()
-        self.assertEqual(b'# HELP cc A counter\n# TYPE cc counter\ncc 1.0\n', generate_latest(self.registry))
+        self.assertEqual(b'''# HELP cc_total A counter
+# TYPE cc_total counter
+cc_total 1.0
+# TYPE cc_created gauge
+cc_created 123.456
+''', generate_latest(self.registry))
+
+    def test_counter_total(self):
+        c = Counter('cc_total', 'A counter', registry=self.registry)
+        c.inc()
+        self.assertEqual(b'''# HELP cc_total A counter
+# TYPE cc_total counter
+cc_total 1.0
+# TYPE cc_created gauge
+cc_created 123.456
+''', generate_latest(self.registry))
 
     def test_gauge(self):
         g = Gauge('gg', 'A gauge', registry=self.registry)
@@ -41,7 +77,13 @@ class TestGenerateText(unittest.TestCase):
     def test_summary(self):
         s = Summary('ss', 'A summary', ['a', 'b'], registry=self.registry)
         s.labels('c', 'd').observe(17)
-        self.assertEqual(b'# HELP ss A summary\n# TYPE ss summary\nss_count{a="c",b="d"} 1.0\nss_sum{a="c",b="d"} 17.0\n', generate_latest(self.registry))
+        self.assertEqual(b'''# HELP ss A summary
+# TYPE ss summary
+ss_count{a="c",b="d"} 1.0
+ss_sum{a="c",b="d"} 17.0
+# TYPE ss_created gauge
+ss_created{a="c",b="d"} 123.456
+''', generate_latest(self.registry))
 
     @unittest.skipIf(sys.version_info < (2, 7), "Test requires Python 2.7+.")
     def test_histogram(self):
@@ -66,17 +108,41 @@ hh_bucket{le="10.0"} 1.0
 hh_bucket{le="+Inf"} 1.0
 hh_count 1.0
 hh_sum 0.05
+# TYPE hh_created gauge
+hh_created 123.456
 ''', generate_latest(self.registry))
 
+    def test_gaugehistogram(self):
+        self.custom_collector(GaugeHistogramMetricFamily('gh', 'help', buckets=[('1.0', 4), ('+Inf', 5)], gsum_value=7))
+        self.assertEqual(b'''# HELP gh help
+# TYPE gh histogram
+gh_bucket{le="1.0"} 4.0
+gh_bucket{le="+Inf"} 5.0
+# TYPE gh_gcount gauge
+gh_gcount 5.0
+# TYPE gh_gsum gauge
+gh_gsum 7.0
+''', generate_latest(self.registry))
+
+    def test_info(self):
+        i = Info('ii', 'A info', ['a', 'b'], registry=self.registry)
+        i.labels('c', 'd').info({'foo': 'bar'})
+        self.assertEqual(b'# HELP ii_info A info\n# TYPE ii_info gauge\nii_info{a="c",b="d",foo="bar"} 1.0\n', generate_latest(self.registry))
+
+    def test_enum(self):
+        i = Enum('ee', 'An enum', ['a', 'b'], registry=self.registry, states=['foo', 'bar'])
+        i.labels('c', 'd').state('bar')
+        self.assertEqual(b'# HELP ee An enum\n# TYPE ee gauge\nee{a="c",b="d",ee="foo"} 0.0\nee{a="c",b="d",ee="bar"} 1.0\n', generate_latest(self.registry))
+
     def test_unicode(self):
-        c = Counter('cc', '\u4500', ['l'], registry=self.registry)
+        c = Gauge('cc', '\u4500', ['l'], registry=self.registry)
         c.labels('\u4500').inc()
-        self.assertEqual(b'# HELP cc \xe4\x94\x80\n# TYPE cc counter\ncc{l="\xe4\x94\x80"} 1.0\n', generate_latest(self.registry))
+        self.assertEqual(b'# HELP cc \xe4\x94\x80\n# TYPE cc gauge\ncc{l="\xe4\x94\x80"} 1.0\n', generate_latest(self.registry))
 
     def test_escaping(self):
-        c = Counter('cc', 'A\ncount\\er', ['a'], registry=self.registry)
-        c.labels('\\x\n"').inc(1)
-        self.assertEqual(b'# HELP cc A\\ncount\\\\er\n# TYPE cc counter\ncc{a="\\\\x\\n\\""} 1.0\n', generate_latest(self.registry))
+        g = Gauge('cc', 'A\ngaug\\e', ['a'], registry=self.registry)
+        g.labels('\\x\n"').inc(1)
+        self.assertEqual(b'# HELP cc A\\ngaug\\\\e\n# TYPE cc gauge\ncc{a="\\\\x\\n\\""} 1.0\n', generate_latest(self.registry))
 
     def test_nonnumber(self):
 
@@ -96,6 +162,29 @@ hh_sum 0.05
         self.registry.register(MyCollector())
         self.assertEqual(b'# HELP nonnumber Non number\n# TYPE nonnumber untyped\nnonnumber 123.0\n', generate_latest(self.registry))
 
+    def test_timestamp(self):
+        class MyCollector(object):
+            def collect(self):
+                metric = Metric("ts", "help", 'untyped')
+                metric.add_sample("ts", {"foo": "a"}, 0, 123.456)
+                metric.add_sample("ts", {"foo": "b"}, 0, -123.456)
+                metric.add_sample("ts", {"foo": "c"}, 0, 123)
+                metric.add_sample("ts", {"foo": "d"}, 0, Timestamp(123, 456000000))
+                metric.add_sample("ts", {"foo": "e"}, 0, Timestamp(123, 456000))
+                metric.add_sample("ts", {"foo": "f"}, 0, Timestamp(123, 456))
+                yield metric
+
+        self.registry.register(MyCollector())
+        self.assertEqual(b'''# HELP ts help
+# TYPE ts untyped
+ts{foo="a"} 0.0 123456
+ts{foo="b"} 0.0 -123456
+ts{foo="c"} 0.0 123000
+ts{foo="d"} 0.0 123456
+ts{foo="e"} 0.0 123000
+ts{foo="f"} 0.0 123000
+''', generate_latest(self.registry))
+
 
 class TestPushGateway(unittest.TestCase):
     def setUp(self):
@@ -209,6 +298,91 @@ class TestPushGateway(unittest.TestCase):
         handler = MetricsHandler.factory(self.registry)
         self.assertEqual(handler.registry, self.registry)
 
+    def test_metrics_handler_subclassing(self):
+        subclass = type(str('MetricsHandlerSubclass'), (MetricsHandler, object), {})
+        handler = subclass.factory(self.registry)
+
+        self.assertTrue(issubclass(handler, (MetricsHandler, subclass)))
+
+
+@pytest.fixture
+def registry():
+    return core.CollectorRegistry()
+
+
+class Collector:
+    def __init__(self, metric_family, *values):
+        self.metric_family = metric_family
+        self.values = values
+
+    def collect(self):
+        self.metric_family.add_metric([], *self.values)
+        return [self.metric_family]
+
+
+def _expect_metric_exception(registry, expected_error):
+    try:
+        generate_latest(registry)
+    except expected_error as exception:
+        assert isinstance(exception.args[-1], core.Metric)
+        # Got a valid error as expected, return quietly
+        return
+
+    raise RuntimeError('Expected exception not raised')
+
+
+@pytest.mark.parametrize('MetricFamily', [
+    core.CounterMetricFamily,
+    core.GaugeMetricFamily,
+])
+@pytest.mark.parametrize('value,error', [
+    (None, TypeError),
+    ('', ValueError),
+    ('x', ValueError),
+    ([], TypeError),
+    ({}, TypeError),
+])
+def test_basic_metric_families(registry, MetricFamily, value, error):
+    metric_family = MetricFamily(MetricFamily.__name__, 'help')
+    registry.register(Collector(metric_family, value))
+    _expect_metric_exception(registry, error)
+
+
+@pytest.mark.parametrize('count_value,sum_value,error', [
+    (None, 0, TypeError),
+    (0, None, TypeError),
+    ('', 0, ValueError),
+    (0, '', ValueError),
+    ([], 0, TypeError),
+    (0, [], TypeError),
+    ({}, 0, TypeError),
+    (0, {}, TypeError),
+])
+def test_summary_metric_family(registry, count_value, sum_value, error):
+    metric_family = core.SummaryMetricFamily('summary', 'help')
+    registry.register(Collector(metric_family, count_value, sum_value))
+    _expect_metric_exception(registry, error)
+
+
+@pytest.mark.parametrize('MetricFamily', [
+    core.HistogramMetricFamily,
+    core.GaugeHistogramMetricFamily,
+])
+@pytest.mark.parametrize('buckets,sum_value,error', [
+    ([('spam', 0), ('eggs', 0)], None, TypeError),
+    ([('spam', 0), ('eggs', None)], 0, TypeError),
+    ([('spam', 0), (None, 0)], 0, AttributeError),
+    ([('spam', None), ('eggs', 0)], 0, TypeError),
+    ([(None, 0), ('eggs', 0)], 0, AttributeError),
+    ([('spam', 0), ('eggs', 0)], '', ValueError),
+    ([('spam', 0), ('eggs', '')], 0, ValueError),
+    ([('spam', ''), ('eggs', 0)], 0, ValueError),
+])
+def test_histogram_metric_families(MetricFamily, registry, buckets, sum_value, error):
+    metric_family = MetricFamily(MetricFamily.__name__, 'help')
+    registry.register(Collector(metric_family, buckets, sum_value))
+    _expect_metric_exception(registry, error)
+
 
 if __name__ == '__main__':
     unittest.main()
